@@ -36,6 +36,21 @@
 #define EXIT_FAILURE 1
 
 int debugmode = 0;
+/* Clickmode: 0 - first finger; 1 - second finger; 2 - center */
+int clickmode = 2;
+
+/* Continuation mode -- when 1, two finger gesture is continued when one finger is released. */
+#define CONTINUATION 1
+
+#if CONTINUATION
+	#define TWO_FINGERS_DOWN fingersDown == 2 && hadTwoFingersOn == 0
+	#define TWO_FINGERS_ON fingersDown > 0 && hadTwoFingersOn == 1
+	#define TWO_FINGERS_UP fingersDown == 0 && hadTwoFingersOn == 1
+#else
+	#define TWO_FINGERS_DOWN fingersDown == 2 && fingersWereDown < 2
+	#define TWO_FINGERS_ON fingersDown == 2
+	#define TWO_FINGERS_UP fingersDown < 2 && fingersWereDown == 2
+#endif
 
 /* Daemonize. Source: http://www-theorie.physik.unizh.ch/~dpotter/howto/daemonize (public domain) */
 static void daemonize(void) {
@@ -365,7 +380,7 @@ void processFingers() {
 
 	if (active == 0)
 		return;
-	if (fingersDown == 2 && fingersWereDown < 2) {
+	if (TWO_FINGERS_DOWN) {
 		/* Second finger touched (and maybe first too) */
 
 		maxDist = 0;
@@ -399,12 +414,22 @@ void processFingers() {
 		XTestFakeMotionEvent(display, -1, gestureStartCenterX,
 				gestureStartCenterY, CurrentTime);
 
-	} else if (fingersDown == 2) {
+	} else if (TWO_FINGERS_ON) {
 		/* Moved with two fingers */
 
-		/* Calculate new center between fingers */
-		currentCenterX = (fingerInfos[0].x + fingerInfos[1].x) / 2;
-		currentCenterY = (fingerInfos[0].y + fingerInfos[1].y) / 2;
+		if(fingersDown == 2) {
+			/* Calculate new center between fingers */
+			currentCenterX = (fingerInfos[0].x + fingerInfos[1].x) / 2;
+			currentCenterY = (fingerInfos[0].y + fingerInfos[1].y) / 2;
+		} else {
+			int i;
+			for(i = 0; i <= 1; i++) {
+				if(fingerInfos[i].down) {
+					currentCenterX = fingerInfos[i].x;
+					currentCenterY = fingerInfos[i].y;
+				}
+			}
+		}
 
 		/* If we are dragScrolling (we are scrolling and there is a brace action,
 		 * we need to move the pointer */
@@ -418,7 +443,7 @@ void processFingers() {
 		while (checkGesture())
 			;
 
-	} else if (fingersDown < 2 && fingersWereDown == 2) {
+	} else if (TWO_FINGERS_UP) {
 		/* Second finger (and maybe also first) released */
 
 		/* If there was a scroll gesture and we have a brace action, perform release. */
@@ -434,6 +459,15 @@ void processFingers() {
 		/* If we haven't performed a gesture and haven't moved too far, perform tap action. */
 		if ((amPerformingGesture == GESTURE_NONE || amPerformingGesture
 				== GESTURE_UNDECIDED) && maxDist < 10) {
+			/* Move pointer to correct position */
+			if(clickmode == 2) {
+				/* We still are at center */
+			} else {
+				/* Assume first finger is at ID 0 and second finger at ID 1, might have to be changed later */
+				XTestFakeMotionEvent(display, -1, fingerInfos[clickmode].x, fingerInfos[clickmode].y,
+						CurrentTime);
+			}
+
 			if (currentProfile->tapInherit) {
 				executeAction(&(defaultProfile.tapAction), EXECUTEACTION_BOTH);
 			} else {
@@ -922,6 +956,12 @@ int main(int argc, char **argv) {
 			debugmode = 1;
 		} else if (strcmp(argv[i], "--wait") == 0) {
 			doWait = 1;
+		} else if (strcmp(argv[i], "--click=first") == 0) {
+			clickmode = 0;
+		} else if (strcmp(argv[i], "--click=second") == 0) {
+			clickmode = 1;
+		} else if (strcmp(argv[i], "--click=center") == 0) {
+			clickmode = 2;
 		} else {
 			devname = argv[i];
 		}
@@ -1121,6 +1161,7 @@ int main(int argc, char **argv) {
 							if (fingerInfos[index].y > displayHeight)
 								fingerInfos[index].y = displayHeight;
 						}
+						fingerInfos[index].down = 1;
 					} else if (0 == ev[i].code) { // Ev_Sync event end
 						/* All finger data received, so process now. */
 						processFingers();
